/*
 * cocoshop
 *
 * Copyright (c) 2011 Andrew
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */

#import "CSTableViewDataSource.h"
#import "CSSprite.h"

@implementation CSTableViewDataSource

@synthesize array=array_;

+ (id)dataSourceWithArray:(NSArray *)anArray
{
	return [[[self alloc] initWithArray:anArray] autorelease];
}

- (id)initWithArray:(NSArray *)anArray
{
	if((self=[super init]))
	{
		[self setArray:anArray];
	}
	
	return self;
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(NSInteger)rowIndex
{
	return [(CSSprite *)[array_ objectAtIndex:rowIndex] name];
}

- (NSInteger)numberOfRowsInTableView:(NSTableView *)aTableView
{
	return [array_ count];
}

- (void)tableView:(NSTableView *)aTableView setObjectValue:(id)anObject forTableColumn:(NSTableColumn *)aTableColumn row:(NSInteger)rowIndex
{
	[(CSSprite *)[array_ objectAtIndex:rowIndex] setName: anObject];
	
	// Update other windows
	NSDictionary *userInfoDict = [NSDictionary dictionaryWithObjectsAndKeys: anObject, @"name", nil];
	[[NSNotificationCenter defaultCenter] postNotificationName:@"didRenameSelectedSprite" object:self userInfo:userInfoDict];
}

- (void)dealloc
{
	[self setArray:nil];
	[super dealloc];
}

@end
